<?php
namespace wcf\system\form\builder\field\poll;
use wcf\data\poll\option\PollOption;
use wcf\system\form\builder\field\AbstractFormField;
use wcf\system\form\builder\TWysiwygFormNode;
use wcf\util\ArrayUtil;
use wcf\util\StringUtil;

/**
 * Represents the form field to manage poll options/answers.
 * 
 * This form field should not be used idenpendently but only via `WysiwygPollFormContainer`.
 * 
 * @author	Matthias Schmidt
 * @copyright	2001-2019 WoltLab GmbH
 * @license	GNU Lesser General Public License <http://opensource.org/licenses/lgpl-license.php>
 * @package	WoltLabSuite\Core\System\Form\Builder\Field\Poll
 * @since	5.2
 */
class PollOptionsFormField extends AbstractFormField {
	use TWysiwygFormNode;
	
	/**
	 * @inheritDoc
	 */
	protected $templateName = '__pollOptionsFormField';
	
	/**
	 * @inheritDoc
	 */
	protected $value = [];
	
	/**
	 * Creates a new instance of `PollOptionsFormField`.
	 */
	public function __construct() {
		$this->label('wcf.poll.options')
			->description('wcf.poll.options.description')
			->addClass('pollOptionContainer');
	}
	
	/**
	 * @inheritDoc
	 */
	public function readValue() {
		if ($this->getDocument()->hasRequestData($this->getPrefixedId()) && is_array($this->getDocument()->getRequestData($this->getPrefixedId()))) {
			$value = array_slice(
				ArrayUtil::trim($this->getDocument()->getRequestData($this->getPrefixedId())),
				0,
				POLL_MAX_OPTIONS
			);
			
			$this->value = [];
			foreach ($value as $showOrder => $option) {
				list($optionID, $optionValue) = explode('_', $option, 2);
				$this->value[$showOrder] = [
					'optionID' => intval($optionID),
					'optionValue' => StringUtil::trim($optionValue)
				];
			}
		}
	}
	
	/**
	 * @inheritDoc
	 */
	public function value($value) {
		$pollOptions = [];
		
		foreach ($value as $pollOption) {
			if ($pollOption instanceof PollOption) {
				$pollOptions[] = [
					'optionID' => $pollOption->optionID,
					'optionValue' => $pollOption->optionValue
				];
			}
			else if (is_array($pollOption) && isset($pollOptions['optionID']) && isset($pollOptions['optionValue'])) {
				$pollOptions[] = [
					'optionID' => $pollOptions['optionID'],
					'optionValue' => $pollOptions['optionValue']
				];
			}
			else {
				throw new \InvalidArgumentException("Given value array contains invalid value of type " . gettype($pollOption) . ".");
			}
		}
		
		return parent::value($value);
	}
	
	/**
	 * @inheritDoc
	 */
	public function validate() {
		parent::validate();
		
		// ensure maximum length that is already validated via JavaScript
		foreach ($this->value as &$value) {
			$value = mb_substr($value, 0, 255);
		}
		unset($value);
	}
}
